@@ -11,7 +11,6 @@
 import copy
 import os
 import time
-from typing import List
 
 from cloudinit import dmi
 from cloudinit import log as logging
@@ -19,8 +18,7 @@ from cloudinit import net, sources
 from cloudinit import url_helper as uhelp
 from cloudinit import util, warnings
 from cloudinit.event import EventScope, EventType
-from cloudinit.net.dhcp import NoDHCPLeaseError
-from cloudinit.net.ephemeral import EphemeralIPNetwork
+from cloudinit.net.dhcp import EphemeralDHCPv4, NoDHCPLeaseError
 from cloudinit.sources.helpers import ec2
 
 LOG = logging.getLogger(__name__)
@@ -69,11 +67,7 @@ class DataSourceEc2(sources.DataSource):
     # Priority ordered list of additional metadata versions which will be tried
     # for extended metadata content. IPv6 support comes in 2016-09-02.
     # Tags support comes in 2021-03-23.
-    extended_metadata_versions: List[str] = [
-        "2021-03-23",
-        "2018-09-24",
-        "2016-09-02",
-    ]
+    extended_metadata_versions = ["2021-03-23", "2018-09-24", "2016-09-02"]
 
     # Setup read_url parameters per get_url_params.
     url_max_wait = 120
@@ -126,16 +120,12 @@ class DataSourceEc2(sources.DataSource):
                 LOG.debug("FreeBSD doesn't support running dhclient with -sf")
                 return False
             try:
-                with EphemeralIPNetwork(
-                    self.fallback_interface, ipv6=True
-                ) as netw:
-                    state_msg = f" {netw.state_msg}" if netw.state_msg else ""
+                with EphemeralDHCPv4(self.fallback_interface):
                     self._crawled_metadata = util.log_time(
                         logfunc=LOG.debug,
-                        msg=f"Crawl of metadata service{state_msg}",
+                        msg="Crawl of metadata service",
                         func=self.crawl_metadata,
                     )
-
             except NoDHCPLeaseError:
                 return False
         else:
@@ -232,7 +222,7 @@ class DataSourceEc2(sources.DataSource):
         else:
             return self.metadata["instance-id"]
 
-    def _maybe_fetch_api_token(self, mdurls):
+    def _maybe_fetch_api_token(self, mdurls, timeout=None, max_wait=None):
         """Get an API token for EC2 Instance Metadata Service.
 
         On EC2. IMDS will always answer an API token, unless
@@ -473,15 +463,21 @@ class DataSourceEc2(sources.DataSource):
         if isinstance(net_md, dict):
             # SRU_BLOCKER: xenial, bionic and eoan should default
             # apply_full_imds_network_config to False to retain original
-            # behavior on those releases.  SRU_UPSTREAM_COMMIT: 6600c6
+            # behavior on those releases.
             result = convert_ec2_metadata_network_config(
                 net_md,
                 fallback_nic=iface,
                 full_network_config=util.get_cfg_option_bool(
-                    self.ds_cfg, "apply_full_imds_network_config", False
+                    self.ds_cfg, "apply_full_imds_network_config", True
                 ),
             )
 
+            # RELEASE_BLOCKER: xenial should drop the below if statement,
+            # because the issue being addressed doesn't exist pre-netplan.
+            # (This datasource doesn't implement check_instance_id() so the
+            # datasource object is recreated every boot; this means we don't
+            # need to modify update_events on cloud-init upgrade.)
+
             # Non-VPC (aka Classic) Ec2 instances need to rewrite the
             # network config file every boot due to MAC address change.
             if self.is_classic_instance():
