@@ -4,12 +4,11 @@ import logging
 import os
 import re
 import subprocess
-import sys
 import tempfile
 from functools import lru_cache
-from typing import Dict, List, NamedTuple, Optional
+from typing import Dict, List, Optional
 
-from uaclient import event_logger, exceptions, gpg, messages, system
+from uaclient import event_logger, exceptions, gpg, messages, util
 
 APT_HELPER_TIMEOUT = 60.0  # 60 second timeout used for apt-helper call
 APT_AUTH_COMMENT = "  # ubuntu-advantage-tools"
@@ -30,9 +29,6 @@ APT_METHOD_HTTPS_FILE = "/usr/lib/apt/methods/https"
 CA_CERTIFICATES_FILE = "/usr/sbin/update-ca-certificates"
 APT_PROXY_CONF_FILE = "/etc/apt/apt.conf.d/90ubuntu-advantage-aptproxy"
 
-APT_UPDATE_SUCCESS_STAMP_PATH = "/var/lib/apt/periodic/update-success-stamp"
-APT_LISTS_PATH = "/var/lib/apt/lists"
-
 # Since we generally have a person at the command line prompt. Don't loop
 # for 5 minutes like charmhelpers because we expect the human to notice and
 # resolve to apt conflict or try again.
@@ -48,11 +44,6 @@ class AptProxyScope(enum.Enum):
     UACLIENT = object()
 
 
-InstalledAptPackages = NamedTuple(
-    "InstalledAptPackages", [("name", str), ("version", str), ("arch", str)]
-)
-
-
 def assert_valid_apt_credentials(repo_url, username, password):
     """Validate apt credentials for a PPA.
 
@@ -68,7 +59,7 @@ def assert_valid_apt_credentials(repo_url, username, password):
         return
     try:
         with tempfile.TemporaryDirectory() as tmpd:
-            system.subp(
+            util.subp(
                 [
                     "/usr/lib/apt/apt-helper",
                     "download-file",
@@ -166,7 +157,7 @@ def run_apt_command(
     :raise UserFacingError: on issues running apt-cache policy.
     """
     try:
-        out, _err = system.subp(
+        out, _err = util.subp(
             cmd, capture=True, retry_sleeps=APT_RETRIES, env=env
         )
     except exceptions.ProcessExecutionError as e:
@@ -190,7 +181,7 @@ def run_apt_command(
 
 
 @lru_cache(maxsize=None)
-def get_apt_cache_policy(
+def run_apt_cache_policy_command(
     error_msg: Optional[str] = None,
     env: Optional[Dict[str, str]] = {},
 ) -> str:
@@ -199,16 +190,6 @@ def get_apt_cache_policy(
     )
 
 
-def get_apt_cache_policy_for_package(
-    package: str,
-    error_msg: Optional[str] = None,
-    env: Optional[Dict[str, str]] = {},
-) -> str:
-    return run_apt_command(
-        cmd=["apt-cache", "policy", package], error_msg=error_msg, env=env
-    )
-
-
 def run_apt_update_command(env: Optional[Dict[str, str]] = {}) -> str:
     try:
         out = run_apt_command(cmd=["apt-get", "update"], env=env)
@@ -225,7 +206,7 @@ def run_apt_update_command(env: Optional[Dict[str, str]] = {}) -> str:
         # Whenever we run an apt-get update command, we must invalidate
         # the existing apt-cache policy cache. Otherwise, we could provide
         # users with incorrect values.
-        get_apt_cache_policy.cache_clear()
+        run_apt_cache_policy_command.cache_clear()
 
     return out
 
@@ -274,7 +255,7 @@ def add_auth_apt_repo(
     except ValueError:  # Then we have a bearer token
         username = "bearer"
         password = credentials
-    series = system.get_platform_info()["series"]
+    series = util.get_platform_info()["series"]
     if repo_url.endswith("/"):
         repo_url = repo_url[:-1]
     assert_valid_apt_credentials(repo_url, username, password)
@@ -313,7 +294,7 @@ def add_auth_apt_repo(
                 maybe_comment=maybe_comment, url=repo_url, suite=suite
             )
         )
-    system.write_file(repo_filename, content)
+    util.write_file(repo_filename, content)
     add_apt_auth_conf_entry(repo_url, username, password)
     source_keyring_file = os.path.join(KEYRINGS_DIR, keyring_file)
     destination_keyring_file = os.path.join(APT_KEYS_DIR, keyring_file)
@@ -324,14 +305,14 @@ def add_apt_auth_conf_entry(repo_url, login, password):
     """Add or replace an apt auth line in apt's auth.conf file or conf.d."""
     apt_auth_file = get_apt_auth_file_from_apt_config()
     _protocol, repo_path = repo_url.split("://")
-    if not repo_path.endswith("/"):  # ensure trailing slash
-        repo_path += "/"
+    if repo_path.endswith("/"):  # strip trailing slash
+        repo_path = repo_path[:-1]
     if os.path.exists(apt_auth_file):
-        orig_content = system.load_file(apt_auth_file)
+        orig_content = util.load_file(apt_auth_file)
     else:
         orig_content = ""
     repo_auth_line = (
-        "machine {repo_path} login {login} password {password}"
+        "machine {repo_path}/ login {login} password {password}"
         "{cmt}".format(
             repo_path=repo_path,
             login=login,
@@ -342,25 +323,23 @@ def add_apt_auth_conf_entry(repo_url, login, password):
     added_new_auth = False
     new_lines = []
     for line in orig_content.splitlines():
-        if not added_new_auth:
-            split_line = line.split()
-            if len(split_line) >= 2:
-                curr_line_repo = split_line[1]
-                if curr_line_repo == repo_path:
-                    # Replace old auth with new auth at same line
-                    new_lines.append(repo_auth_line)
-                    added_new_auth = True
-                    continue
-                if curr_line_repo in repo_path:
-                    # Insert our repo before.
-                    # We are a more specific apt repo match
-                    new_lines.append(repo_auth_line)
-                    added_new_auth = True
+        machine_match = re.match(r"machine\s+(?P<repo_url>[.\-\w]+)/?.*", line)
+        if machine_match:
+            matched_repo = machine_match.group("repo_url")
+            if matched_repo == repo_path:
+                # Replace old auth with new auth at same line
+                new_lines.append(repo_auth_line)
+                added_new_auth = True
+                continue
+            if matched_repo in repo_path:
+                # Insert our repo before. We are a more specific apt repo match
+                new_lines.append(repo_auth_line)
+                added_new_auth = True
         new_lines.append(line)
     if not added_new_auth:
         new_lines.append(repo_auth_line)
     new_lines.append("")
-    system.write_file(apt_auth_file, "\n".join(new_lines), mode=0o600)
+    util.write_file(apt_auth_file, "\n".join(new_lines), mode=0o600)
 
 
 def remove_repo_from_apt_auth_file(repo_url):
@@ -370,58 +349,61 @@ def remove_repo_from_apt_auth_file(repo_url):
         repo_path = repo_path[:-1]
     apt_auth_file = get_apt_auth_file_from_apt_config()
     if os.path.exists(apt_auth_file):
-        apt_auth = system.load_file(apt_auth_file)
+        apt_auth = util.load_file(apt_auth_file)
         auth_prefix = "machine {repo_path}/ login".format(repo_path=repo_path)
         content = "\n".join(
             [line for line in apt_auth.splitlines() if auth_prefix not in line]
         )
         if not content:
-            system.remove_file(apt_auth_file)
+            os.unlink(apt_auth_file)
         else:
-            system.write_file(apt_auth_file, content, mode=0o600)
+            util.write_file(apt_auth_file, content, mode=0o600)
 
 
 def remove_auth_apt_repo(
-    repo_filename: str, repo_url: str, keyring_file: Optional[str] = None
+    repo_filename: str, repo_url: str, keyring_file: str = None
 ) -> None:
     """Remove an authenticated apt repo and credentials to the system"""
-    system.remove_file(repo_filename)
+    util.del_file(repo_filename)
     if keyring_file:
         keyring_file = os.path.join(APT_KEYS_DIR, keyring_file)
-        system.remove_file(keyring_file)
+        util.del_file(keyring_file)
     remove_repo_from_apt_auth_file(repo_url)
 
 
 def restore_commented_apt_list_file(filename: str) -> None:
     """Uncomment commented deb lines in the given file."""
     if os.path.exists(filename):
-        file_content = system.load_file(filename)
+        file_content = util.load_file(filename)
         file_content = file_content.replace("# deb ", "deb ")
-        system.write_file(filename, file_content)
+        util.write_file(filename, file_content)
 
 
 def add_ppa_pinning(apt_preference_file, repo_url, origin, priority):
     """Add an apt preferences file and pin for a PPA."""
+    series = util.get_platform_info()["series"]
     _protocol, repo_path = repo_url.split("://")
     if repo_path.endswith("/"):  # strip trailing slash
         repo_path = repo_path[:-1]
     content = (
         "Package: *\n"
-        "Pin: release o={origin}\n"
-        "Pin-Priority: {priority}\n".format(origin=origin, priority=priority)
+        "Pin: release o={origin}, n={series}\n"
+        "Pin-Priority: {priority}\n".format(
+            origin=origin, priority=priority, series=series
+        )
     )
-    system.write_file(apt_preference_file, content)
+    util.write_file(apt_preference_file, content)
 
 
 def get_apt_auth_file_from_apt_config():
     """Return to patch to the system configured APT auth file."""
-    out, _err = system.subp(
+    out, _err = util.subp(
         ["apt-config", "shell", "key", APT_CONFIG_AUTH_PARTS_DIR]
     )
     if out:  # then auth.conf.d parts is present
         return out.split("'")[1] + "90ubuntu-advantage"
     else:  # then use configured /etc/apt/auth.conf
-        out, _err = system.subp(
+        out, _err = util.subp(
             ["apt-config", "shell", "key", APT_CONFIG_AUTH_FILE]
         )
         return out.split("'")[1].rstrip("/")
@@ -433,9 +415,7 @@ def find_apt_list_files(repo_url, series):
     if repo_path.endswith("/"):  # strip trailing slash
         repo_path = repo_path[:-1]
     lists_dir = "/var/lib/apt/lists"
-    out, _err = system.subp(
-        ["apt-config", "shell", "key", APT_CONFIG_LISTS_DIR]
-    )
+    out, _err = util.subp(["apt-config", "shell", "key", APT_CONFIG_LISTS_DIR])
     if out:  # then lists dir is present in config
         lists_dir = out.split("'")[1]
 
@@ -452,7 +432,8 @@ def find_apt_list_files(repo_url, series):
 def remove_apt_list_files(repo_url, series):
     """Remove any apt list files present for this repo_url and series."""
     for path in find_apt_list_files(repo_url, series):
-        system.remove_file(path)
+        if os.path.exists(path):
+            os.unlink(path)
 
 
 def clean_apt_files(*, _entitlements=None):
@@ -474,41 +455,19 @@ def clean_apt_files(*, _entitlements=None):
             continue
         repo_file = ent_cls.repo_list_file_tmpl.format(name=ent_cls.name)
         pref_file = ent_cls.repo_pref_file_tmpl.format(name=ent_cls.name)
+
         if os.path.exists(repo_file):
-            event.info(
-                "Removing apt source file: {}".format(repo_file),
-                file_type=sys.stderr,
-            )
-            system.remove_file(repo_file)
+            logging.info("Removing apt source file: %s", repo_file)
+            os.unlink(repo_file)
+
         if os.path.exists(pref_file):
-            event.info(
-                "Removing apt preferences file: {}".format(pref_file),
-                file_type=sys.stderr,
-            )
-            system.remove_file(pref_file)
+            logging.info("Removing apt preferences file: %s", pref_file)
+            os.unlink(pref_file)
 
 
-def is_installed(pkg: str) -> bool:
-    return pkg in get_installed_packages_names()
-
-
-def get_installed_packages() -> List[InstalledAptPackages]:
-    out, _ = system.subp(["apt", "list", "--installed"])
-    package_list = out.splitlines()[1:]
-    return [
-        InstalledAptPackages(
-            name=entry.split("/")[0],
-            version=entry.split(" ")[1],
-            arch=entry.split(" ")[2],
-        )
-        for entry in package_list
-    ]
-
-
-def get_installed_packages_names(include_versions: bool = False) -> List[str]:
-    package_list = get_installed_packages()
-    pkg_names = [pkg.name for pkg in package_list]
-    return pkg_names
+def get_installed_packages() -> List[str]:
+    out, _ = util.subp(["dpkg-query", "-W", "--showformat=${Package}\\n"])
+    return out.splitlines()
 
 
 def setup_apt_proxy(
@@ -562,52 +521,6 @@ def setup_apt_proxy(
         apt_proxy_config = messages.APT_PROXY_CONFIG_HEADER + apt_proxy_config
 
     if apt_proxy_config == "":
-        system.remove_file(APT_PROXY_CONF_FILE)
+        util.remove_file(APT_PROXY_CONF_FILE)
     else:
-        system.write_file(APT_PROXY_CONF_FILE, apt_proxy_config)
-
-
-def setup_unauthenticated_repo(
-    repo_filename: str,
-    repo_pref_filename: str,
-    repo_url: str,
-    keyring_file: str,
-    apt_origin: str,
-    suites: List[str],
-) -> None:
-    content = "# Written by ubuntu-advantage-tools"
-    for suite in suites:
-        content += "\n"
-        content += messages.REPO_FILE_CONTENT.format(
-            url=repo_url, apt_suite=suite
-        )
-    system.write_file(repo_filename, content)
-    source_keyring_file = os.path.join(KEYRINGS_DIR, keyring_file)
-    destination_keyring_file = os.path.join(APT_KEYS_DIR, keyring_file)
-    gpg.export_gpg_key(source_keyring_file, destination_keyring_file)
-    add_ppa_pinning(
-        repo_pref_filename,
-        repo_url,
-        apt_origin,
-        "never",
-    )
-
-
-def compare_versions(version1: str, version2: str, relation: str) -> bool:
-    """Return True comparing version1 to version2 with the given relation."""
-    try:
-        system.subp(
-            ["dpkg", "--compare-versions", version1, relation, version2]
-        )
-        return True
-    except exceptions.ProcessExecutionError:
-        return False
-
-
-def get_apt_cache_time() -> Optional[float]:
-    cache_time = None
-    if os.path.exists(APT_UPDATE_SUCCESS_STAMP_PATH):
-        cache_time = os.stat(APT_UPDATE_SUCCESS_STAMP_PATH).st_mtime
-    elif os.path.exists(APT_LISTS_PATH):
-        cache_time = os.stat(APT_LISTS_PATH).st_mtime
-    return cache_time
+        util.write_file(APT_PROXY_CONF_FILE, apt_proxy_config)
