@@ -1,8 +1,9 @@
-from typing import Optional, Tuple  # noqa: F401
+from typing import Optional, Tuple, Union  # noqa: F401
 
 from uaclient import util
-from uaclient.config import update_ua_messages
 from uaclient.entitlements import repo
+from uaclient.entitlements.entitlement_status import CanDisableFailure
+from uaclient.jobs.update_messaging import update_apt_and_motd_messages
 
 
 class ESMBaseEntitlement(repo.RepoEntitlement):
@@ -12,14 +13,16 @@ class ESMBaseEntitlement(repo.RepoEntitlement):
     def _perform_enable(self, silent: bool = False) -> bool:
         enable_performed = super()._perform_enable(silent=silent)
         if enable_performed:
-            update_ua_messages(self.cfg)
+            update_apt_and_motd_messages(self.cfg)
         return enable_performed
 
-    def disable(self, silent=False) -> bool:
-        disable_performed = super().disable(silent=silent)
+    def disable(
+        self, silent=False
+    ) -> Tuple[bool, Union[None, CanDisableFailure]]:
+        disable_performed, fail = super().disable(silent=silent)
         if disable_performed:
-            update_ua_messages(self.cfg)
-        return disable_performed
+            update_apt_and_motd_messages(self.cfg)
+        return disable_performed, fail
 
 
 class ESMAppsEntitlement(ESMBaseEntitlement):
@@ -32,10 +35,8 @@ class ESMAppsEntitlement(ESMBaseEntitlement):
 
     @property
     def repo_pin_priority(self) -> Optional[str]:
-        """All LTS with the exception of Trusty should pin esm-apps."""
+        """All LTS should pin esm-apps."""
         series = util.get_platform_info()["series"]
-        if series == "trusty":
-            return None
 
         if self.valid_service:
             if util.is_lts(series):
@@ -44,10 +45,8 @@ class ESMAppsEntitlement(ESMBaseEntitlement):
 
     @property
     def disable_apt_auth_only(self) -> bool:
-        """All LTSexcept Trusty remove APT auth files upon disable"""
+        """All LTS remove APT auth files upon disable"""
         series = util.get_platform_info()["series"]
-        if series == "trusty":
-            return False
 
         if self.valid_service:
             return util.is_lts(series)
