@@ -4,27 +4,47 @@ import logging
 import os
 import re
 import socket
+import subprocess
 import sys
 import time
+import uuid
 from contextlib import contextmanager
-from functools import wraps
+from errno import ENOENT
+from functools import lru_cache, wraps
 from http.client import HTTPMessage
-from typing import Any, Dict, List, Mapping, Optional, Tuple, Union
+from typing import (
+    Any,
+    Dict,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Union,
+)
 from urllib import error, request
 from urllib.parse import urlparse
 
 from uaclient import event_logger, exceptions, messages
-from uaclient.defaults import CONFIG_FIELD_ENVVAR_ALLOWLIST
 from uaclient.types import MessagingOperations
 
+REBOOT_FILE_CHECK_PATH = "/var/run/reboot-required"
+REBOOT_PKGS_FILE_PATH = "/var/run/reboot-required.pkgs"
+ETC_MACHINE_ID = "/etc/machine-id"
+DBUS_MACHINE_ID = "/var/lib/dbus/machine-id"
 DROPPED_KEY = object()
 
+# N.B. this relies on the version normalisation we perform in get_platform_info
+REGEX_OS_RELEASE_VERSION = r"(?P<release>\d+\.\d+) (LTS )?\((?P<series>\w+).*"
+
 UA_NO_PROXY_URLS = ("169.254.169.254", "metadata", "[fd00:ec2::254]")
 PROXY_VALIDATION_APT_HTTP_URL = "http://archive.ubuntu.com"
 PROXY_VALIDATION_APT_HTTPS_URL = "https://esm.ubuntu.com"
 PROXY_VALIDATION_SNAP_HTTP_URL = "http://api.snapcraft.io"
 PROXY_VALIDATION_SNAP_HTTPS_URL = "https://api.snapcraft.io"
 
+OVERRIDE_SELECTOR_WEIGHTS = {"series_overrides": 1, "series": 2, "cloud": 3}
 
 event = event_logger.get_event_logger()
 
@@ -74,9 +94,7 @@ class DatetimeAwareJSONDecoder(json.JSONDecoder):
         for key, value in o.items():
             if isinstance(value, str):
                 try:
-                    new_value = parse_rfc3339_date(
-                        value
-                    )  # type: Union[str, datetime.datetime]
+                    new_value = parse_rfc3339_date(value)
                 except ValueError:
                     # This isn't a string containing a valid ISO 8601 datetime
                     new_value = value
@@ -84,6 +102,93 @@ class DatetimeAwareJSONDecoder(json.JSONDecoder):
         return o
 
 
+def _get_override_weight(
+    override_selector: Dict[str, str], selector_values: Dict[str, str]
+) -> int:
+    override_weight = 0
+    for selector, value in override_selector.items():
+        if (selector, value) not in selector_values.items():
+            return 0
+        override_weight += OVERRIDE_SELECTOR_WEIGHTS[selector]
+
+    return override_weight
+
+
+def _select_overrides(
+    entitlement: Dict[str, Any], series_name: str, cloud_type: str
+) -> Dict[int, Dict[str, Any]]:
+    overrides = {}
+
+    selector_values = {"series": series_name, "cloud": cloud_type}
+
+    series_overrides = entitlement.pop("series", {}).pop(series_name, {})
+    if series_overrides:
+        overrides[
+            OVERRIDE_SELECTOR_WEIGHTS["series_overrides"]
+        ] = series_overrides
+
+    general_overrides = entitlement.pop("overrides", [])
+    for override in general_overrides:
+        weight = _get_override_weight(
+            override.pop("selector"), selector_values
+        )
+        if weight:
+            overrides[weight] = override
+
+    return overrides
+
+
+def apply_contract_overrides(
+    orig_access: Dict[str, Any], series: Optional[str] = None
+) -> None:
+    """Apply series-specific overrides to an entitlement dict.
+
+    This function mutates orig_access dict by applying any series-overrides to
+    the top-level keys under 'entitlement'. The series-overrides are sparse
+    and intended to supplement existing top-level dict values. So, sub-keys
+    under the top-level directives, obligations and affordance sub-key values
+    will be preserved if unspecified in series-overrides.
+
+    To more clearly indicate that orig_access in memory has already had
+    the overrides applied, the 'series' key is also removed from the
+    orig_access dict.
+
+    :param orig_access: Dict with original entitlement access details
+    """
+    from uaclient.clouds.identity import get_cloud_type
+
+    if not all([isinstance(orig_access, dict), "entitlement" in orig_access]):
+        raise RuntimeError(
+            'Expected entitlement access dict. Missing "entitlement" key:'
+            " {}".format(orig_access)
+        )
+
+    series_name = get_platform_info()["series"] if series is None else series
+    cloud_type, _ = get_cloud_type()
+    orig_entitlement = orig_access.get("entitlement", {})
+
+    overrides = _select_overrides(orig_entitlement, series_name, cloud_type)
+
+    for _weight, overrides_to_apply in sorted(overrides.items()):
+        for key, value in overrides_to_apply.items():
+            current = orig_access["entitlement"].get(key)
+            if isinstance(current, dict):
+                # If the key already exists and is a dict,
+                # update that dict using the override
+                current.update(value)
+            else:
+                # Otherwise, replace it wholesale
+                orig_access["entitlement"][key] = value
+
+
+def del_file(path: str) -> None:
+    try:
+        os.unlink(path)
+    except OSError as e:
+        if e.errno != ENOENT:
+            raise e
+
+
 @contextmanager
 def disable_log_to_console():
     """
@@ -190,6 +295,126 @@ def get_dict_deltas(
     return deltas
 
 
+@lru_cache(maxsize=None)
+def get_machine_id(cfg) -> str:
+    """Get system's unique machine-id or create our own in data_dir."""
+    # Generate, cache our own uuid if not present in config or on the system
+
+    if cfg.machine_token:
+        cfg_machine_id = cfg.machine_token.get("machineTokenInfo", {}).get(
+            "machineId"
+        )
+        if cfg_machine_id:
+            return cfg_machine_id
+
+    fallback_machine_id_file = cfg.data_path("machine-id")
+
+    for path in [ETC_MACHINE_ID, DBUS_MACHINE_ID, fallback_machine_id_file]:
+        if os.path.exists(path):
+            content = load_file(path).rstrip("\n")
+            if content:
+                return content
+    machine_id = str(uuid.uuid4())
+    write_file(fallback_machine_id_file, machine_id)
+    return machine_id
+
+
+@lru_cache(maxsize=None)
+def get_platform_info() -> Dict[str, str]:
+    """
+    Returns a dict of platform information.
+
+    N.B. This dict is sent to the contract server, which requires the
+    distribution, type and release keys.
+    """
+    os_release = parse_os_release()
+    platform_info = {
+        "distribution": os_release.get("NAME", "UNKNOWN"),
+        "type": "Linux",
+    }
+
+    version = os_release["VERSION"]
+    # Strip off an LTS point release (20.04.1 LTS -> 20.04 LTS)
+    version = re.sub(r"\.\d LTS", " LTS", version)
+    platform_info["version"] = version
+
+    match = re.match(REGEX_OS_RELEASE_VERSION, version)
+    if not match:
+        raise RuntimeError(
+            "Could not parse /etc/os-release VERSION: {} (modified to"
+            " {})".format(os_release["VERSION"], version)
+        )
+    match_dict = match.groupdict()
+    platform_info.update(
+        {
+            "release": match_dict["release"],
+            "series": match_dict["series"].lower(),
+        }
+    )
+
+    uname = os.uname()
+    platform_info["kernel"] = uname.release
+    out, _err = subp(["dpkg", "--print-architecture"])
+    platform_info["arch"] = out.strip()
+
+    return platform_info
+
+
+@lru_cache(maxsize=None)
+def is_lts(series: str) -> bool:
+    out, _err = subp(["/usr/bin/ubuntu-distro-info", "--supported-esm"])
+    return series in out
+
+
+@lru_cache(maxsize=None)
+def is_current_series_lts() -> bool:
+    series = get_platform_info()["series"]
+    return is_lts(series)
+
+
+@lru_cache(maxsize=None)
+def is_active_esm(series: str) -> bool:
+    """Return True when Ubuntu series supports ESM and is actively in ESM."""
+    if not is_lts(series):
+        return False
+    out, _err = subp(
+        ["/usr/bin/ubuntu-distro-info", "--series", series, "-yeol"]
+    )
+    return int(out) <= 0
+
+
+@lru_cache(maxsize=None)
+def is_container(run_path: str = "/run") -> bool:
+    """Checks to see if this code running in a container of some sort"""
+
+    # We may mistake schroot environments for containers by just relying
+    # in the other checks present in that function. To guarantee that
+    # we do not identify a schroot as a container, we are explicitly
+    # using the 'ischroot' command here.
+    try:
+        subp(["ischroot"])
+        return False
+    except exceptions.ProcessExecutionError:
+        pass
+
+    try:
+        subp(["systemd-detect-virt", "--quiet", "--container"])
+        return True
+    except (IOError, OSError):
+        pass
+
+    for filename in ("container_type", "systemd/container"):
+        path = os.path.join(run_path, filename)
+        if os.path.exists(path):
+            return True
+    return False
+
+
+def is_exe(path: str) -> bool:
+    # return boolean indicating if path exists and is executable.
+    return os.path.isfile(path) and os.access(path, os.X_OK)
+
+
 def is_service_url(url: str) -> bool:
     try:
         parsed_url = urlparse(url)
@@ -207,6 +432,26 @@ def is_service_url(url: str) -> bool:
     return True
 
 
+def load_file(filename: str, decode: bool = True) -> str:
+    """Read filename and decode content."""
+    logging.debug("Reading file: %s", filename)
+    with open(filename, "rb") as stream:
+        content = stream.read()
+    return content.decode("utf-8")
+
+
+@lru_cache(maxsize=None)
+def parse_os_release(release_file: Optional[str] = None) -> Dict[str, str]:
+    if not release_file:
+        release_file = "/etc/os-release"
+    data = {}
+    for line in load_file(release_file).splitlines():
+        key, value = line.split("=", 1)
+        if value:
+            data[key] = value.strip().strip('"')
+    return data
+
+
 def prompt_choices(msg: str = "", valid_choices: List[str] = []) -> str:
     """Interactive prompt message, returning a valid choice from msg.
 
@@ -345,7 +590,153 @@ def readurl(
     return content, resp.headers
 
 
-def is_config_value_true(config: Dict[str, Any], path_to_value: str) -> bool:
+def _subp(
+    args: Sequence[str],
+    rcs: Optional[List[int]] = None,
+    capture: bool = False,
+    timeout: Optional[float] = None,
+    env: Optional[Dict[str, str]] = None,
+) -> Tuple[str, str]:
+    """Run a command and return a tuple of decoded stdout, stderr.
+
+    @param args: A list of arguments to feed to subprocess.Popen
+    @param rcs: A list of allowed return_codes. If returncode not in rcs
+        raise a ProcessExecutionError.
+    @param capture: Boolean set True to log the command and response.
+    @param timeout: Optional float indicating number of seconds to wait for
+        subp to return.
+    @param env: Optional dictionary of environment variable to pass to Popen.
+
+    @return: Tuple of utf-8 decoded stdout, stderr
+    @raises ProcessExecutionError on invalid command or returncode not in rcs.
+    @raises subprocess.TimeoutError when timeout specified and the command
+        exceeds that number of seconds.
+    """
+    bytes_args = [
+        x if isinstance(x, bytes) else x.encode("utf-8") for x in args
+    ]
+    if env:
+        env.update(os.environ)
+    if rcs is None:
+        rcs = [0]
+    redacted_cmd = redact_sensitive_logs(" ".join(args))
+    try:
+        proc = subprocess.Popen(
+            bytes_args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env
+        )
+        (out, err) = proc.communicate(timeout=timeout)
+    except OSError:
+        try:
+            raise exceptions.ProcessExecutionError(
+                cmd=redacted_cmd,
+                exit_code=proc.returncode,
+                stdout=out.decode("utf-8"),
+                stderr=err.decode("utf-8"),
+            )
+        except UnboundLocalError:
+            raise exceptions.ProcessExecutionError(cmd=redacted_cmd)
+    if proc.returncode not in rcs:
+        raise exceptions.ProcessExecutionError(
+            cmd=redacted_cmd,
+            exit_code=proc.returncode,
+            stdout=out.decode("utf-8"),
+            stderr=err.decode("utf-8"),
+        )
+    if capture:
+        logging.debug(
+            "Ran cmd: %s, rc: %s stderr: %s",
+            redacted_cmd,
+            proc.returncode,
+            err,
+        )
+    return out.decode("utf-8"), err.decode("utf-8")
+
+
+def subp(
+    args: Sequence[str],
+    rcs: Optional[List[int]] = None,
+    capture: bool = False,
+    timeout: Optional[float] = None,
+    retry_sleeps: Optional[List[float]] = None,
+    env: Optional[Dict[str, str]] = None,
+) -> Tuple[str, str]:
+    """Run a command and return a tuple of decoded stdout, stderr.
+
+     @param subp: A list of arguments to feed to subprocess.Popen
+     @param rcs: A list of allowed return_codes. If returncode not in rcs
+         raise a ProcessExecutionError.
+     @param capture: Boolean set True to log the command and response.
+     @param timeout: Optional float indicating number of seconds to wait for a
+         subp call to return.
+     @param retry_sleeps: Optional list of sleep lengths to apply between
+        retries. Specifying a list of [0.5, 1] instructs subp to retry twice
+        on failure; sleeping half a second before the first retry and 1 second
+        before the next retry.
+     @param env: Optional dictionary of environment variables to provide to
+        subp.
+
+    @return: Tuple of utf-8 decoded stdout, stderr
+    @raises ProcessExecutionError on invalid command or returncode not in rcs.
+    @raises subprocess.TimeoutError when timeout specified and the command
+        exceeds that number of seconds.
+    """
+    retry_sleeps = retry_sleeps.copy() if retry_sleeps is not None else None
+    while True:
+        try:
+            out, err = _subp(args, rcs, capture, timeout, env=env)
+            break
+        except exceptions.ProcessExecutionError as e:
+            if capture:
+                logging.debug(redact_sensitive_logs(str(e)))
+                msg = "Stderr: {}\nStdout: {}".format(e.stderr, e.stdout)
+                logging.warning(redact_sensitive_logs(msg))
+            if not retry_sleeps:
+                raise
+            retry_msg = " Retrying %d more times." % len(retry_sleeps)
+            logging.debug(redact_sensitive_logs(str(e) + retry_msg))
+            time.sleep(retry_sleeps.pop(0))
+    return out, err
+
+
+def which(program: str) -> Optional[str]:
+    """Find whether the provided program is executable in our PATH"""
+    if os.path.sep in program:
+        # if program had a '/' in it, then do not search PATH
+        if is_exe(program):
+            return program
+    paths = [
+        p.strip('"') for p in os.environ.get("PATH", "").split(os.pathsep)
+    ]
+    normalized_paths = [os.path.abspath(p) for p in paths]
+    for path in normalized_paths:
+        program_path = os.path.join(path, program)
+        if is_exe(program_path):
+            return program_path
+    return None
+
+
+def write_file(filename: str, content: str, mode: int = 0o644) -> None:
+    """Write content to the provided filename encoding it if necessary.
+
+    @param filename: The full path of the file to write.
+    @param content: The content to write to the file.
+    @param mode: The filesystem mode to set on the file.
+    """
+    logging.debug("Writing file: %s", filename)
+    with open(filename, "wb") as fh:
+        fh.write(content.encode("utf-8"))
+        fh.flush()
+    os.chmod(filename, mode)
+
+
+def remove_file(file_path: str) -> None:
+    """Remove a file if it exists, logging a message about removal."""
+    if os.path.exists(file_path):
+        logging.debug("Removing file: %s", file_path)
+        os.unlink(file_path)
+
+
+def is_config_value_true(config: Dict[str, Any], path_to_value: str):
     """Check if value parameter can be translated into a boolean 'True' value.
 
     @param config: A config dict representing
@@ -402,9 +793,6 @@ REDACT_SENSITIVE_LOGS = [
     r"(\"identityToken\": \")[^\"]+",
     r"(response:\s+http://metadata/computeMetadata/v1/instance/"
     "service-accounts.*data: ).*",
-    r"(\'token\': \')[^\']+",
-    r"(\'userCode\': \')[^\']+",
-    r"(\'magic_token=)[^\']+",
 ]
 
 
@@ -418,6 +806,68 @@ def redact_sensitive_logs(
     return redacted_log
 
 
+def should_reboot(
+    installed_pkgs: Optional[Set[str]] = None,
+    installed_pkgs_regex: Optional[Set[str]] = None,
+) -> bool:
+    """Check if the system needs to be rebooted.
+
+    :param installed_pkgs: If provided, verify if the any packages in
+        the list are present on /var/run/reboot-required.pkgs. If that
+        param is provided, we will only return true if we have the
+        reboot-required marker file and any package in reboot-required.pkgs
+        file. When both installed_pkgs and installed_pkgs_regex are
+        provided, they act as an OR, so only one of the two lists must have
+        a match to return True.
+    :param installed_pkgs_regex: If provided, verify if the any regex in
+        the list matches any line in /var/run/reboot-required.pkgs. If that
+        param is provided, we will only return true if we have the
+        reboot-required marker file and any match in reboot-required.pkgs
+        file. When both installed_pkgs and installed_pkgs_regex are
+        provided, they act as an OR, so only one of the two lists must have
+        a match to return True.
+    """
+
+    # If the reboot marker file doesn't exist, we don't even
+    # need to look at the installed_pkgs param
+    if not os.path.exists(REBOOT_FILE_CHECK_PATH):
+        return False
+
+    # If there is no installed_pkgs to check, we will rely only
+    # on the existence of the reboot marker file
+    if installed_pkgs is None and installed_pkgs_regex is None:
+        return True
+
+    try:
+        reboot_required_pkgs = set(
+            load_file(REBOOT_PKGS_FILE_PATH).split("\n")
+        )
+    except FileNotFoundError:
+        # If the file doesn't exist, we will default to the
+        # reboot  marker file
+        return True
+
+    if installed_pkgs is not None:
+        if len(installed_pkgs.intersection(reboot_required_pkgs)) != 0:
+            return True
+
+    if installed_pkgs_regex is not None:
+        for pkg_name in reboot_required_pkgs:
+            for pkg_regex in installed_pkgs_regex:
+                if re.search(pkg_regex, pkg_name):
+                    return True
+
+    return False
+
+
+def is_installed(package_name: str) -> bool:
+    try:
+        out, _ = subp(["dpkg", "-l", package_name])
+        return "ii  {} ".format(package_name) in out
+    except exceptions.ProcessExecutionError:
+        return False
+
+
 def handle_message_operations(
     msg_ops: Optional[MessagingOperations],
 ) -> bool:
@@ -546,78 +996,3 @@ def handle_unicode_characters(message: str) -> str:
         message = message.replace(messages.FAIL_X + " ", "")
 
     return message
-
-
-def get_pro_environment():
-    return {
-        k: v
-        for k, v in os.environ.items()
-        if k.lower() in CONFIG_FIELD_ENVVAR_ALLOWLIST
-        or k.startswith("UA_FEATURES")
-        or k == "UA_CONFIG_FILE"
-    }
-
-
-def depth_first_merge_overlay_dict(base_dict, overlay_dict):
-    """Merge the contents of overlay dict into base_dict not only on top-level
-    keys, but on all on the depths of the overlay_dict object. For example,
-    using these values as entries for the function:
-
-    base_dict = {"a": 1, "b": {"c": 2, "d": 3}}
-    overlay_dict = {"b": {"c": 10}}
-
-    Should update base_dict into:
-
-    {"a": 1, "b": {"c": 10, "d": 3}}
-
-    @param base_dict: The dict to be updated
-    @param overlay_dict: The dict with information to be added into base_dict
-    """
-
-    def update_dict_list(base_values, overlay_values, key):
-        merge_id_key_map = {
-            "availableResources": "name",
-            "resourceEntitlements": "type",
-        }
-        values_to_append = []
-        id_key = merge_id_key_map.get(key)
-        for overlay_value in overlay_values:
-            was_replaced = False
-            for base_value_idx, base_value in enumerate(base_values):
-                if base_value.get(id_key) == overlay_value.get(id_key):
-                    depth_first_merge_overlay_dict(base_value, overlay_value)
-                    was_replaced = True
-
-            if not was_replaced:
-                values_to_append.append(overlay_value)
-
-        base_values.extend(values_to_append)
-
-    for key, value in overlay_dict.items():
-        base_value = base_dict.get(key)
-        if isinstance(base_value, dict) and isinstance(value, dict):
-            depth_first_merge_overlay_dict(base_dict[key], value)
-        elif isinstance(base_value, list) and isinstance(value, list):
-            if len(base_value) and isinstance(base_value[0], dict):
-                update_dict_list(base_dict[key], value, key=key)
-            else:
-                """
-                Most other lists which aren't lists of dicts are lists of
-                strs. Replace that list # with the overlay value."""
-                base_dict[key] = value
-        else:
-            base_dict[key] = value
-
-
-def deduplicate_arches(arches: List[str]) -> List[str]:
-    deduplicated_arches = set()
-    arch_aliases = {
-        "x86_64": "amd64",
-        "i686": "i386",
-        "ppc64le": "ppc64el",
-        "aarch64": "arm64",
-        "armv7l": "armhf",
-    }
-    for arch in arches:
-        deduplicated_arches.add(arch_aliases.get(arch.lower(), arch))
-    return sorted(list(deduplicated_arches))
